/*
Copyright 2011- Hamidreza Chitsaz (chitsaz@wayne.edu)

    This file is part of Camel.

    Camel is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Camel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Camel; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/***************************************************************************
 * Title:          kmers.h 
 * Author:         Hamidreza Chitsaz
 * Created:        2011
 * Last modified:  10/10/2011
 *
 * Copyright (c) 2011- Hamidreza Chitsaz
 * All Rights Reserved
 * See file LICENSE for details.
 ***************************************************************************/

#ifndef KMERS_H
#define KMERS_H

#include <pthread.h>

#include "kmer.h"

template<class Data>
struct KmerData
{
	Kmer first;
	Data second;
}
#if PACK_MEMORY
__attribute__((__packed__))
#endif
;


template<class Data, template<class Pair> class Container>
class Kmers {
public:
	Kmers();
	~Kmers();
	void insert(KmerData<Data> &);
	bool fetch(Kmer &, Data &);
	void fetch(Kmer &,  KmerData<Data> *&);
	size_t size() {return kmersNum;}
	size_t hash(Kmer &kmer) {return kmerhash(kmer);}
	size_t count();
	void setThreadSafe(bool t);
	void setContainersThreadSafe(bool t);

	void begin(); 
	KmerData<Data>& next(); 
	bool hasNext();
private:
	bool threadsafe;
	size_t kmersNum;
	Container<KmerData<Data> > **table;
	size_t idx, spIndex;
	pthread_mutex_t *gmutex;
};

#endif

